
/*
  writes stream into a file. And takes care of a few things
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */


extern "C" {
#include <yafcore/streamWriter.h>
	   }

#include <yafcore/yafRuntime.defs>

#include <yafcore/runtimeTableYAF.h>
#include <yafcore/commandTable.h>
#include <yafcore/outputInterface.h>

#ifdef LINUX
#include <yafcore/audioLinux.h>
#endif

#include <yafxplayer/inputDecoderXPlayer.h>
#include <yafxplayer/commandTableXPlayer.h>

#include <pthread.h>

//#define DEBUG


// This include should stay here. But ....
#include <yafxplayer/runtimeTableXPlayer.h>

static int _fd;
static char* _filename;
static int _nBytesWrite;
static CommandTable* _runtimeTable;
static int _blockedArea; 
static int lDevice=false;
static long bytes;
static OutputInterface* output;

#ifdef LINUX
static AudioLinux* audioLinux=NULL;
#endif

extern "C" void StreamWriter_writeMessage(char* msg) {
  output->setNr(0);
  output->appendBuffer(msg);
  output->flushBuffer();
}


extern "C" void StreamWriter_Construct(void* out) {
  _fd=-2;
  _filename=NULL;
  _nBytesWrite=0;
  output=(OutputInterface*)out;
  _runtimeTable=new CommandTable();
  _runtimeTable->join(new RuntimeTableXPlayer());
  _blockedArea=false;

#ifdef LINUX
  audioLinux=new AudioLinux();
#endif
  StreamWriter_internalDevice(lDevice);
  bytes=0;
}



  
extern "C" void StreamWriter_Destruct() {
#ifdef LINUX
  if (audioLinux != NULL) {
    delete audioLinux;
  }
#endif
}


void StreamWriter_setBytesCounter(long aValue) {
  bytes=aValue;
}


long StreamWriter_getBytesCounter() {
  return bytes;
}

  

void StreamWriter_setStreamFile(char* filename) {
  StreamWriter_closeStream();
  _filename=filename;
}


extern "C" int StreamWriter_openStream() {
  _fd=open(_filename,O_WRONLY | O_TRUNC);

  return _fd;
}

extern "C" int StreamWriter_isOpenStream() {
  return (_fd>=0);
}


extern "C" int StreamWriter_closeStream() {
  int ret=0;
  if (_fd >= 0) {
    ret=close(_fd);
    cout << "closed streamwriter:"<<ret<<endl;
    _fd=-2;
    _filename=NULL;
    StreamWriter_resetStream();
  }
  return ret;
}


extern "C" int StreamWriter_isBlocked() {
  return _blockedArea;
}



extern "C" int StreamWriter_writeStream(char* buf, int len) {
  int ret=0;
#ifdef LINUX
  if (audioLinux != NULL) {
    if (audioLinux->isOpen()) {
      audioLinux->write(buf,len);
    }
  }
#endif
  
  if (_fd > 0) {
#ifdef DEBUG
    cout <<"Command:1 Msg: vor write ........"<<len <<endl;
#endif

    ret=write(_fd,buf,len);
#ifdef DEBUG
    cout <<"Command:1 Msg: nach write ........"<<ret <<endl;
#endif    
    if (ret < 0) {
      
      StreamWriter_closeStream();
      StreamWriter_writeMessage("fifo error in streamWriter");
      if (errno == EAGAIN) {
	StreamWriter_writeMessage("errno:EAGAIN\n");
      }
      if (errno == EINTR) {
	StreamWriter_writeMessage("errno:EINTR\n");
      }
      if (errno == EIO) {
	StreamWriter_writeMessage("errno:EIO\n");
      }
      if (errno == EBADF) {
	StreamWriter_writeMessage("errno:EBADF\n");
      }    
      if (errno == EINVAL) {
	StreamWriter_writeMessage("errno:EINVAL\n");
      }  
      if (errno == EISDIR) {
	StreamWriter_writeMessage("errno:EISDIR\n");
      }  
      if (errno == EFAULT) {
	StreamWriter_writeMessage("errno:EFAULT\n");
      } 
      exit(-1);
      return ret;
    }

#ifdef DEBUG
    if (ret < len) {
      cout <<"Command:1 Msg:write errror"<<ret<<" len: "<<len<<endl; 
    }
#endif
    bytes=bytes+len;
    return len;
  }
  bytes=bytes+len;
  return _fd;
}


extern "C" void StreamWriter_resetStream() {
  _nBytesWrite=0;
}



extern "C" int StreamWriter_getBytesWrite() {
  return _nBytesWrite;
}


extern "C" void StreamWriter_sendStreamInfo(int channels, 
					    int sampleSize,
					    int speed) {
  StreamWriter_sendChannels(channels);
  StreamWriter_sendSampleSize(sampleSize);
  StreamWriter_sendSpeed(speed);
#ifdef LINUX
  if (audioLinux != NULL) {
    audioLinux->init(channels,sampleSize,speed);
  }
#endif
}

extern "C" void StreamWriter_sendChannels(int channels) {
  char* cmd=_runtimeTable->getCommand(_PLAYER_RUN_CHANNELS);
  printf("Command:0 Msg:%s %d %d\n",
	 cmd,
	 channels,
	 _nBytesWrite+1);
}

extern "C" void StreamWriter_sendSampleSize(int size) {
  char* cmd=_runtimeTable->getCommand(_PLAYER_RUN_SAMPLESIZE);
  printf("Command:0 Msg:%s %d %d\n",
	 cmd,
	 size,
	 _nBytesWrite+1);
}


extern "C" void StreamWriter_sendSpeed(int speed) {
  char* cmd=_runtimeTable->getCommand(_PLAYER_RUN_SPEED);
  printf("Command:0 Msg:%s %d %d\n",
	 cmd,
	 speed,
	 _nBytesWrite+1);
}




extern "C" void StreamWriter_internalDevice(int on) {
#ifdef LINUX  
  if (lDevice == true) {
    if (audioLinux != NULL) {
      if (audioLinux->isOpen()) {
	audioLinux->close();
	delete audioLinux;
	audioLinux=NULL;
      }
    }
  }
  lDevice=on;
  if (lDevice == true) {
    if (audioLinux != NULL) {
      audioLinux->open();
    }
  }
#endif


}


