//
//  kmpg
//
//  Copyright (C) 1997 Bjarni Ivarsson
//  email: Bjarni.Ivarsson@swipnet.se
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#ifndef LEDPANEL_H
#define LEDPANEL_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qpushbt.h>

#include <kslider.h>

#include "fileinfo.h"
#include <yafBridge/frontendDecoder.h>
#include "playlistmgr.h"
#include "trackcalendar.h"
#include "infoline.h"
#include "statusline.h"

class LedPanel : public QWidget
{
      Q_OBJECT
   public:
      LedPanel( QWidget*, PlaylistMgr* , FrontendDecoder*);
      ~LedPanel();
      enum TimeMode {track, trackrem, total, totalrem};
      TimeMode mode, timedisplay;
      void ledSetTime(TimeMode , int , int , int , int );
      
      void showTrackCal( bool );
      void showSlider( bool );
      void showControls( bool );
      void setField ( int, QString );
      
   private:
      int value_digits[12];
      QPixmap pm_digits[10], pm_dots;
      QPixmap pm_blank, pm_pause, pm_play, pm_panel, pm_minus;
      QPixmap pm_back;
      int _expanded;
      TrackCalendar *_trackCalendar;
      InfoLine *_infoLine;
      StatusLine *_statusLine;
      PlaylistMgr *_mgr;
      FrontendDecoder *_decoder;
      Fileinfo _finfo;

      // The slider
      QWidget *_sliderPanel;
      KSlider *_slider;

      // Controls
      QWidget *_controlPanel;
      QPushButton *_pbPlay, *_pbPause, *_pbStop, *_pbFwd, *_pbBwd, *_pbNext, *_pbPrev;
      bool _showTrackCal, _showSlider, _showControls;

      
      void loadPixmaps();
      void updateDigit( int, int );
      void paintEvent( QPaintEvent* );
      void styleChange( GUIStyle );

      // for time display
      int track_played, track_total, list_played, list_total, current_frame, current_sec, total_sec;
      bool _update;

   protected:
      virtual void mouseDoubleClickEvent( QMouseEvent* );
      virtual void mousePressEvent( QMouseEvent* );

   public slots:
      void play();
      void pause();
      void stop();
      void forward();
      void rewind();
      void setTrack( int );
      void setListStatus( bool );
      void setLeds( int, int );
      void frame( int );
      void setInfo( Fileinfo& );
      void doSizeChanged();
      void enableUpdate();
      void disableUpdate();
      void decoderTime(float);
      void sliderFrame(int);
      void ledPanelTimeMode(bool change);
      void StatusClicked(int);
      void doplay();
      void seekTo(int frame);    
      void dopause();          // hack for statusdevice

  signals:
      void sizeChanged();
      void timeDisplayChanged(bool);
      void seek(int);

};

#endif
